// These will be added programmatically.
// Uncomment them temporarily to check compilation.

//#define ALBEDO_TINT
//#define ALPHA_TEST

//#define DUAL_PASS
//#define ENVIRONMENT_MAP

//#define FOG

//#define LIGHT_MAP

//#define SPECULAR



uniform mat4 u_ViewProj;
uniform mat4 u_View;
uniform vec2 u_TexAnim;
uniform vec4 u_AlbedoTint;
uniform float u_TexScale;

#ifdef FOG
uniform float u_FogNearPlane;
uniform float u_FogRcpRange;
uniform float u_FogOpacity;
#endif

#ifdef SPECULAR

uniform mediump float u_SpecularPower;

uniform mediump vec3 u_SpecularColour;

uniform mediump vec3 u_SpecularDir;

#endif

#ifdef ENVIRONMENT_MAP

uniform highp vec3 u_CameraPosition;

#endif



attribute vec4 a_Position;
attribute vec2 a_Texcoord;

#ifdef LIGHT_MAP

attribute vec2 a_Texcoord2;

#endif

#if defined(ENVIRONMENT_MAP) || defined(SPECULAR)

attribute vec3 a_Normal;

#endif// defined(ENVIRONMENT_MAP) || defined(SPECULAR)

attribute mediump vec4 a_Colour;

varying lowp vec4 v_VertexColour;
varying mediump vec2 v_TexCoord;

#ifdef LIGHT_MAP

varying mediump vec2 v_LightMapUV;

#endif

#ifdef FOG

varying lowp float v_Fog;

#endif

#ifdef SPECULAR

varying lowp vec3 v_Specular;

#endif

#ifdef ENVIRONMENT_MAP

varying mediump vec3 v_Reflection;

#endif

#ifdef ALBEDO_TINT

#define TINT u_AlbedoTint

#else	

const mediump vec4 c_Two = vec4(2.0, 2.0, 2.0, 1.0);

#define TINT c_Two

#endif// ALBEDO_TINT





void main()
{
    gl_Position = u_ViewProj * a_Position;

	

	#if defined(FOG) || defined(SPECULAR) || defined(ENVIRONMENT_MAP) 

    vec4 viewpos = u_View * a_Position;

    float viewdist = length(viewpos.xyz);

	#endif

    

	#ifdef FOG

    float fogdepth = clamp((viewdist - u_FogNearPlane) * u_FogRcpRange, 0.0, 1.0);

	v_Fog = u_FogOpacity * fogdepth;

	#endif// FOG

	

	v_TexCoord = a_Texcoord * u_TexScale + u_TexAnim;

	

	#ifdef LIGHT_MAP

	v_LightMapUV = a_Texcoord2  * u_TexScale;



	#ifdef DUAL_PASS

	v_VertexColour = vec4(TINT.xyz, a_Colour.a);

	#else//!DUAL_PASS

	v_VertexColour = TINT;

	#endif//!DUAL_PASS



	#else//!LIGHT_MAP



	v_VertexColour = a_Colour * TINT;



	#endif//!LIGHT_MAP


	#ifdef ENVIRONMENT_MAP

	mediump vec3 camdir = normalize(a_Position.xyz - u_CameraPosition);

	v_Reflection = reflect(camdir, a_Normal);

	#endif	

	#ifdef SPECULAR

	mediump vec3 normal = (u_View * vec4(a_Normal, 0.0)).xyz;

	mediump vec3 viewdir = viewpos.xyz / -viewdist;

	mediump vec3 halfdir = normalize(u_SpecularDir + viewdir);

	mediump float ndoth = dot(normal, halfdir);

	v_Specular = max(0.0, pow(ndoth, u_SpecularPower)) * u_SpecularColour;

	#endif

}

