// These will be added programmatically.
// Uncomment them temporarily to check compilation.
//#define ALPHA_TEST
//#define DUAL_PASS
//#define ENVIRONMENT_MAP
//#define FOG
//#define LIGHT_MAP
//#define NORMAL_MAP
//#define REFLECTIVITY_MASK
//#define SPECULAR
//#define SPECULAR_MAP
//#define SELF_ILLUMINATE
//#define SELF_ILLUMINATE_MAP

precision lowp float;

uniform sampler2D s_Albedo;
uniform sampler2D s_Alpha1;
#ifdef DUAL_PASS
uniform sampler2D s_Albedo2;
uniform sampler2D s_Alpha2;
#endif
#ifdef NORMAL_MAP
uniform sampler2D s_Normal;
#endif
#ifdef LIGHT_MAP
uniform sampler2D s_LightMap;
#endif
#ifdef ENVIRONMENT_MAP
uniform samplerCube s_Environment;
#endif

#ifdef SPECULAR
uniform mediump float u_SpecularPower;
uniform mediump vec3 u_SpecularColour;
#endif
#ifdef FOG
uniform vec3 u_FogColour;
#endif
#ifdef SELF_ILLUMINATE
uniform vec4 u_SelfIllumination;
#endif
#ifdef ENVIRONMENT_MAP
uniform float u_Reflectivity;
#endif

varying vec4 v_VertexColour;
varying mediump vec2 v_TexCoord;
#ifdef LIGHT_MAP
varying mediump vec2 v_LightMapUV;
#endif
#ifdef FOG
varying float v_Fog;
#endif
#ifdef SPECULAR
// Tangent space vectors

#ifdef NORMAL_MAP
//varying mediump vec3 v_LightDir;
varying mediump vec3 v_ViewDir;
#else// NORMAL_MAP
varying lowp vec3 v_Specular;
#endif// !NORMAL_MAP

#endif
#ifdef ENVIRONMENT_MAP
varying mediump vec3 v_Reflection;
#endif

void main()
{	
	#ifdef LIGHT_MAP
		// Note that vertex colour has albedo tint baked into it.
		vec4 light_colour = texture2D(s_LightMap, v_LightMapUV) * v_VertexColour;
	#else
		vec4 light_colour = v_VertexColour;
	#endif
	vec4 albedo1 = texture2D(s_Albedo, v_TexCoord);
	// TODO - optimise out if unused?
	albedo1.a = texture2D(s_Alpha1, v_TexCoord).r;

	#ifdef ENVIRONMENT_MAP
	{
		float fReflectivity = u_Reflectivity;
	
		#ifdef REFLECTIVITY_MASK
			fReflectivity *= albedo1.a;
		#endif// REFLECTIVITY_MASK
		
		albedo1.xyz = mix(albedo1.xyz, textureCube(s_Environment, v_Reflection).xyz, fReflectivity);
	}
	#endif//ENVIRONMENT_MAP	
		
	#ifdef DUAL_PASS
		vec4 albedo2 = texture2D(s_Albedo2, v_TexCoord);
		// TODO - optimise out if unused?
		albedo2.a = texture2D(s_Alpha2, v_TexCoord).r;
		vec4 colour = mix(albedo2, albedo1, v_VertexColour.w) * light_colour;
	#else//! DUAL_PASS

		#ifdef SELF_ILLUMINATE
			#ifdef SELF_ILLUMINATE_MAP
				vec4 colour = albedo1 * (light_colour + u_SelfIllumination * albedo1.a);	
			#else//!SELF_ILLUMINATE_MAP
				vec4 colour = albedo1 * (light_colour + u_SelfIllumination);
			#endif//!SELF_ILLUMINATE_MAP
	
		#else//!SELF_ILLUMINATE
			vec4 colour = albedo1 * light_colour;
		#endif//!SELF_ILLUMINATE
	
	#endif//!DUAL_PASS
	
	#ifdef ALPHA_TEST
	if(colour.a <= 0.05)
	{
		discard;
	}
	#endif

	#ifdef SPECULAR

	#ifdef NORMAL_MAP
	mediump vec3 normal = normalize(texture2D(s_Normal, v_TexCoord).xyz * 2.0 - 1.0);

	// Stick a light at the camera for now
	mediump vec3 viewdir = normalize(v_ViewDir);
	mediump float ndotv = dot(normal, viewdir);

	//mediump vec3 lightdir = normalize(v_LightDir);
	//mediump vec3 halfdir = normalize(lightdir + viewdir);
	//mediump float ndoth = dot(normal, halfdir);
	//mediump float ndotl = dot(normal, lightdir);
	
	mediump float specular = max(0.0, pow(ndotv, u_SpecularPower));

	#ifdef SPECULAR_MAP
	specular *= colour.a;
	#endif//SPECULAR_MAP

	colour.xyz += u_SpecularColour * specular;

	#else// NORMAL_MAP
	
	#ifdef SPECULAR_MAP
	colour.xyz += v_Specular * colour.a;
	#else// SPECULAR_MAP
	colour.xyz += v_Specular;
	#endif//SPECULAR_MAP
	
	#endif// !NORMAL_MAP
	
	#endif//SPECULAR

	#ifdef FOG
	colour.xyz = mix(colour.xyz, u_FogColour, v_Fog);
	#endif

	gl_FragColor = colour;
}

