precision mediump float;

//#extension GL_OES_standard_derivatives : enable



uniform sampler2D s_tex0;



uniform vec4 u_OutlineColour;

uniform vec4 u_OuterGlowColour;



uniform vec2 u_TextureSize;

uniform float u_OuterGlowMin;

uniform float u_OuterGlowMax;



uniform float u_OutlineHalfThickness;

uniform float u_AntiAliasDistance;

uniform float u_InnerAlpha;

uniform float u_Tolerance;



uniform bool u_Outline;

uniform bool u_AntiAlias;

uniform bool u_OuterGlow;



varying vec4 v_VertexColour;

varying vec2 v_TexCoord;



/******************************************************************************/

/* Functions                                                                  */

/******************************************************************************/



float saturate(const float fIn)

{

	return clamp(fIn, 0.0, 1.0);

}



void main()

{

	// Use the difference in size of texture compared to size of screen.

	// In otherwords if the texel to screen ratio is 1:1 then fSizeBias should be 1.0

	// If the glyph is drawn at half the size then the fSizeBias should be 2.0;

	// If the glyph is drawn twice the size of its texture then fSizeBias is a 0.5f.

	// This can be used to scale the alpha blending edge size based on the scale of the glyph.

	// The smaller the glyph the larger the alpha blending range needs to be.

	

	// KOR - While this is nice in theory, it's barely noticeable and halves the speed of the fragment shader.

//	vec2 spacing = dFdx(v_TexCoord) * u_TextureSize;

//	float fSizeBias = length(spacing);

//	float fAlphaDist = saturate( u_AntiAliasDistance * fSizeBias );

	float fAlphaDist = u_AntiAliasDistance;



	vec4 xBaseColour = v_VertexColour;



	float fHardEdge = u_Tolerance;



	float fDist = texture2D(s_tex0, v_TexCoord).a;



	if ( u_Outline )

	{

		float fOutlineFadeOff = fAlphaDist;

		float fOutlineHalfSize = u_OutlineHalfThickness + fOutlineFadeOff;

	

		float fOutlineMax = fHardEdge + fOutlineHalfSize;



		if ( fDist <= fOutlineMax )

		{



			vec4 xOutlineColour = u_OutlineColour * v_VertexColour.a;

			// fHardEdge represents the value at which the edge of the lettering

			// occurs and the blending with the background needs to start

			fHardEdge = u_Tolerance - u_OutlineHalfThickness;



			float fFactor = 1.0;



			if ( fDist >= u_Tolerance )

			{

				// When the outline is merged with the inside of the letter blend the

				// outline colour with the letter colour. e.g. blend black outline with white text.

				fFactor = smoothstep(fOutlineMax, fOutlineMax - fOutlineFadeOff, fDist);

				xBaseColour = mix(xBaseColour, xOutlineColour, fFactor);

			}

			else

			{

				// When the pixel is outside of the letter just set its colour to the outline colour.

				// If anti-aliasing is whiched on then this will be alpha blended with the background.

				xBaseColour = xOutlineColour;

			}

		}

	}



	float fBlend = 1.0;



	if ( u_AntiAlias )

	{

		// Alpha blend the pixel with the background. fHardEdge represents the min point where

		// alpha blending should occur. Any distance value between fMinEdge & fMaxEdge will be

		// scaled between 0.0 and 1.0;

		//float fMinEdge = saturate(fHardEdge - fAlphaDist);

		//float fMaxEdge = saturate(fHardEdge + fAlphaDist);

		//fBlend *= smoothstep(fMinEdge,fMaxEdge,fDist);

		

		// KOR - smoothstep is overkill, let's just do a linear blend.

		// Technically we should be clamping here, but the hardware will clamp alpha for us.

		// TODO - subtract alpha dist from hard edge in uniform to save a cycle.

		fBlend *= (fDist + fAlphaDist - fHardEdge) / (fAlphaDist * 2.0);

	}

	else

	{

		// If alpha blending is not enabled then fHardEdge represents the cutoff where the pixel will

		// be drawn.

		float fTest = float(fDist >= fHardEdge);

		fBlend *= fTest;

	}



	if ( u_OuterGlow )

	{

		// As we are no longer guaranteeing that 0 <= fBlend <= 1 in the antialiasing,

		// clamp it here so we don't get weird mix results.

		fBlend = clamp(fBlend, 0.0, 1.0);

		

		// If an outer glow is need then use the distance and scale it between

		// u_OuterGlowMin and 0.5f.

		// Modulate the glow colour by the result.



		// fGlowFactor is used to modulate the glow colour so it fade off as fDist

		// approaches u_OuterGlowMin. If fDist is less than u_OuterGlowMin then

		// the alpha value of the glow colour is 0. If fDist is greater than

		// u_OuterGlowMax the alpha value is 1.

		float fGlowFactor = smoothstep(u_OuterGlowMin, u_OuterGlowMax, fDist);



		vec4 xOuterGlowColour = u_OuterGlowColour * v_VertexColour.a;



		vec4 xGlowc = xOuterGlowColour;

		xGlowc.a *= fGlowFactor;



		// Lerp between the modulated colour and the base colour using the blend value.

		// If the alpha blend is disabled then the blend value is 1 for an fDist >= 0.5 otherwise

		// its 0. In other words either the base colour of the glyph is used or the modulated

		// glow colour.

		// If alpha blending is on then the glyph colour and outglow colours are blended together

		// between the two alpha blending min/max values.



		xBaseColour *= u_InnerAlpha;



		xBaseColour = mix(xGlowc, xBaseColour, fBlend );

	}

	else

	{

		// Just use the alpha from previous calculations.

		xBaseColour.a *= fBlend;

	}



	gl_FragColor = xBaseColour;

}

