// These will be added programmatically.

// Uncomment them temporarily to check compilation.

//#define ALBEDO_TINT

//#define ALPHA_TEST

//#define ENVIRONMENT_MAP

//#define FOG

//#define SPECULAR



attribute vec4 position;

attribute vec3 normal;

attribute vec2 texcoord;



uniform float u_Alpha;

uniform vec4 u_SphericalHarmonic[3];

uniform mat4 u_View;

uniform mat4 u_ViewProj;

uniform vec2 u_TexAnim;

#ifdef ENVIRONMENT_MAP

uniform highp vec3 u_CameraPosition;

uniform mat4 u_BoneMatrix;

#endif

#ifdef FOG

uniform float u_FogNearPlane;

uniform float u_FogRcpRange;

uniform float u_FogOpacity;

#endif



#ifdef SPECULAR

uniform mediump float u_SpecularPower;

uniform mediump vec3 u_SpecularColour;

#endif



varying vec4 v_VertexColour;

varying vec2 v_TexCoord;

#ifdef FOG

varying float v_Fog;

#endif

#ifdef SPECULAR

varying lowp vec3 v_Specular;

#endif

#ifdef ENVIRONMENT_MAP

varying mediump vec3 v_Reflection;

#endif

vec3 SHLookup( const vec4 xNormal, const vec4 xRedSH, const vec4 xGreenSH, const vec4 xBlueSH )

{

	vec3 xColour;



	xColour.r = dot( xNormal, xRedSH );

	xColour.g = dot( xNormal, xGreenSH );

	xColour.b = dot( xNormal, xBlueSH );



	xColour = max( xColour, 0.0 );

	

	return xColour;

}



void main()

{

    gl_Position = u_ViewProj * position;

    

	vec4 ViewPos = u_View * position;



    float depth = length(ViewPos.xyz);



	vec3 ViewNorm = (u_View * vec4(normal, 0.0)).xyz;

	vec3 ToCamera = ViewPos.xyz / -depth;

	float LightIntensity = dot(ViewNorm, ToCamera);



	vec3 LightColour = SHLookup( vec4( normal, 1.0 ),  u_SphericalHarmonic[0],  u_SphericalHarmonic[1],  u_SphericalHarmonic[2] );



	v_VertexColour = vec4(LightColour, u_Alpha);

	

	v_TexCoord = texcoord + u_TexAnim;



	#ifdef FOG

    float fog_depth = clamp((depth - u_FogNearPlane) * u_FogRcpRange, 0.0, 1.0);

    v_Fog = u_FogOpacity * fog_depth;

	#endif// FOG

	#ifdef ENVIRONMENT_MAP

	// Need worldspace position and normal.

	mediump vec3 camdir = normalize( (u_BoneMatrix * position).xyz - u_CameraPosition);

	v_Reflection = reflect(camdir, (u_BoneMatrix * vec4(normal, 0.0)).xyz);

	#endif	

	#ifdef SPECULAR

	mediump vec3 viewdir = ViewPos.xyz / -depth;

	mediump float ndotv = dot(ViewNorm, viewdir);

	v_Specular = max(0.0, pow(ndotv, u_SpecularPower)) * u_SpecularColour;

	#endif

}

