// These will be added programmatically.

// Uncomment them temporarily to check compilation.

//#define ALBEDO_TINT

//#define ALPHA_TEST

//#define ENVIRONMENT_MAP

//#define FOG

//#define NORMAL_MAP

//#define SPECULAR

//#define SPECULAR_MAP



attribute vec4 a_BoneWeights;

attribute vec4 a_BoneIndices;

attribute vec2 a_TexCoord;

attribute vec3 a_Normal;

#ifdef NORMAL_MAP

attribute vec3 a_Tangent;

attribute vec3 a_Binormal;

#endif

attribute vec4 a_Position;



#ifdef FOG

uniform float u_FogNearPlane;

uniform float u_FogRcpRange;

uniform float u_FogOpacity;

#endif



#ifdef SPECULAR

#ifndef NORMAL_MAP

uniform mediump float u_SpecularPower;

uniform mediump vec3 u_SpecularColour;

#endif

#endif



uniform vec4 u_BoneMatrices[3 * 36];

uniform float u_Alpha;

uniform vec4 u_SphericalHarmonic[3];

uniform mat4 u_ViewProj;

uniform mat4 u_View;

uniform vec2 u_TexAnim;

#ifdef ENVIRONMENT_MAP
uniform highp vec3 u_CameraPosition;
#endif



varying vec2 v_TexCoord;

varying vec4 v_VertexColour;

#ifdef FOG

varying float v_Fog;

#endif

#ifdef SPECULAR

#ifdef NORMAL_MAP

// Tangent space vectors

//varying vec3 v_LightDir;

varying vec3 v_ViewDir;

#else

varying lowp vec3 v_Specular;

#endif

#endif

#ifdef ENVIRONMENT_MAP
varying mediump vec3 v_Reflection;
#endif


vec3 SHLookup( const vec4 xNormal, const vec4 xRedSH, const vec4 xGreenSH, const vec4 xBlueSH )

{

	vec3 xColour;



	xColour.r = dot( xNormal, xRedSH );

	xColour.g = dot( xNormal, xGreenSH );

	xColour.b = dot( xNormal, xBlueSH );



	xColour = max( xColour, 0.0 );

	

	return xColour;

}



void main()

{

	// TODO - worth using vec3s for intermediate positions?

	vec4 temp_pos1;

	vec3 world_normal;



	int index_1 = int(a_BoneIndices.x);



	vec4 bone1row1 = u_BoneMatrices[index_1];

	vec4 bone1row2 = u_BoneMatrices[index_1 + 1];

	vec4 bone1row3 = u_BoneMatrices[index_1 + 2];



	temp_pos1.x = dot (a_Position, bone1row1);

	temp_pos1.y = dot (a_Position, bone1row2);

	temp_pos1.z = dot (a_Position, bone1row3);

	temp_pos1.w = 1.0;

	

	world_normal.x = dot (a_Normal, bone1row1.xyz);

	world_normal.y = dot (a_Normal, bone1row2.xyz);	

	world_normal.z = dot (a_Normal, bone1row3.xyz);	



	vec3 LightColour = SHLookup( vec4( world_normal, 1.0 ),  u_SphericalHarmonic[0],  u_SphericalHarmonic[1],  u_SphericalHarmonic[2] );

	

	vec3 view_normal = (u_View * vec4(world_normal, 0.0)).xyz;

	#ifdef NORMAL_MAP

	vec3 tangent, binormal;

	tangent.x = dot (a_Tangent, bone1row1.xyz);

	tangent.y = dot (a_Tangent, bone1row2.xyz);	

	tangent.z = dot (a_Tangent, bone1row3.xyz);	

	binormal.x = dot (a_Binormal, bone1row1.xyz);

	binormal.y = dot (a_Binormal, bone1row2.xyz);	

	binormal.z = dot (a_Binormal, bone1row3.xyz);	

	tangent = (u_View * vec4(tangent.x, tangent.y, tangent.z, 0.0)).xyz;

	binormal = (u_View * vec4(binormal.x, binormal.y, binormal.z, 0.0)).xyz;

	

	//vec3 lightdir = vec3(0.0, -1.0, 0.0);

	mat3 tangentframe = mat3(tangent.x, binormal.x, view_normal.x,

							 tangent.y, binormal.y, view_normal.y,

							 tangent.z, binormal.z, view_normal.z);

	//v_LightDir = tangentframe * lightdir;

	#endif	



	vec4 world_pos;

	if(a_BoneWeights.y > 0.0)

	{

		int index_2 = int(a_BoneIndices.y);

		vec4 temp_pos2;

		temp_pos2.x = dot (a_Position, u_BoneMatrices[index_2]);

		temp_pos2.y = dot (a_Position, u_BoneMatrices[index_2 + 1]);

		temp_pos2.z = dot (a_Position, u_BoneMatrices[index_2 + 2]);

		temp_pos2.w = 1.0;

		

		if(a_BoneWeights.z > 0.0)

		{

			int index_3 = int(a_BoneIndices.z);

			vec4 temp_pos3;

			temp_pos3.x = dot (a_Position, u_BoneMatrices[index_3]);

			temp_pos3.y = dot (a_Position, u_BoneMatrices[index_3 + 1]);

			temp_pos3.z = dot (a_Position, u_BoneMatrices[index_3 + 2]);

			temp_pos3.w = 1.0;

			

			if(a_BoneWeights.w > 0.0)

			{

				int index_4 = int(a_BoneIndices.w);

				vec4 temp_pos4;

				temp_pos4.x = dot (a_Position, u_BoneMatrices[index_4]);

				temp_pos4.y = dot (a_Position, u_BoneMatrices[index_4 + 1]);

				temp_pos4.z = dot (a_Position, u_BoneMatrices[index_4 + 2]);

				temp_pos4.w = 1.0;

				

				world_pos = ((((temp_pos1 * a_BoneWeights.x) + (temp_pos2 * a_BoneWeights.y)) + (temp_pos3 * a_BoneWeights.z)) + (temp_pos4 * a_BoneWeights.w));

			}

			else

			{

				world_pos = (((temp_pos1 * a_BoneWeights.x) + (temp_pos2 * a_BoneWeights.y)) + (temp_pos3 * a_BoneWeights.z));

			}

		}

		else

		{

			//world_pos = ((temp_pos1 * a_BoneWeights.x) + (temp_pos2 * a_BoneWeights.y));

			// Take advantage of weights adding up to 1.

			world_pos = mix(temp_pos2, temp_pos1, a_BoneWeights.x);

		}

	}

	else

	{

		world_pos = temp_pos1;

	}



	vec4 view_pos = (u_View * world_pos);

	

	// TODO - use projection trick from PowerVR SGX OpenGL ES 2.0

	// Application Development Recommendations 7.7.1s

	gl_Position = (u_ViewProj * world_pos);



	float depth = length(view_pos.xyz);

	#ifdef FOG

    float fog_depth = clamp((depth - u_FogNearPlane) * u_FogRcpRange, 0.0, 1.0);

    v_Fog = u_FogOpacity * fog_depth;

	#endif// FOG



	vec3 view_dir = view_pos.xyz / -depth;

	#ifdef ENVIRONMENT_MAP
		mediump vec3 camdir = normalize(world_pos.xyz - u_CameraPosition);
		v_Reflection = reflect(camdir, world_normal);
	#endif	


	#ifdef SPECULAR

	

	#ifdef NORMAL_MAP

	v_ViewDir = tangentframe * view_dir;

	#else

	mediump float ndotv = dot(view_normal, view_dir);

	v_Specular = max(0.0, pow(ndotv, u_SpecularPower)) * u_SpecularColour;	

	#endif	



	#endif//SPECULAR	



	v_TexCoord = a_TexCoord + u_TexAnim;

	

	v_VertexColour.xyz = LightColour;

	v_VertexColour.w = u_Alpha;

}

